// GenKeyTok Utility
//
// This little program takes a public key for an assembly
// as input in text form (as displayed by ildasm for example),
// converts the ASCII representation of the key into a
// binary key file, and then spawns sn.exe to have it display
// the public key token for that key.
//
// The input data can be extracted from a text file or from
// the clipboard.  Whitespace (including newlines) are removed,
// but there cannot be any other non-whitespace characters in
// the input data.
//
// This utility requires that on sn.exe be on the PATH in order
// to operate.
//
// Mike Woodring
// http://staff.develop.com/woodring
//
using System;
using System.Reflection;
using System.Windows.Forms;
using System.Text;
using System.IO;
using System.Diagnostics;

class GenKeyTokenUtility
{
    static void Main( string[] args )
    {
        if( args.Length != 1 )
        {
            Console.WriteLine("Usage:");
            Console.WriteLine("genkeytok infile (process text file)");
            Console.WriteLine("genkeytok /c     (process clipboard)");
            return;
        }

        try
        {
            string keyText;
            
            // Grab the 'cleansed' input data from the clipboard
            // or from the specified file.
            //
            if( string.Compare(args[0], "/c", true) == 0 )
            {
                keyText = FetchTextFromClipboard();
            }
            else
            {
                keyText = FetchTextFromFile(args[0]);
            }

            if( keyText.Length > 0 )
            {
                string keyFile = Path.GetTempFileName();

                try
                {
                    // Save the text-encoded key to a proper binary file like
                    // the kind sn.exe produces when you use "sn -p privpub.snk pub.snk"
                    // to extract the public key from a key pair file.
                    //
                    SaveAsBinary(keyText, keyFile);

                    // Launch sn.exe to process the newly created binary file.
                    //
                    ProcessStartInfo psi = new ProcessStartInfo("sn.exe", string.Format("-tp {0}", keyFile));
                    psi.UseShellExecute = false;
                    Process proc = Process.Start(psi);
                    proc.WaitForExit();
                }
                finally
                {
                    File.Delete(keyFile);
                }
            }
        }
        catch( Exception e )
        {
            Console.WriteLine("An exception occurred processing input data.");
            Console.WriteLine(e.Message);
        }
    }

    static char[] nibbleChars = { '0', '1', '2', '3', '4', '5', '6', '7',
                                  '8', '9', 'a', 'b', 'c', 'd', 'e', 'f' };

    static byte CharsToByte( char hiNibble, char lowNibble )
    {
        byte b = 0;

        hiNibble = char.ToLower(hiNibble);
        lowNibble = char.ToLower(lowNibble);

        if( Array.IndexOf(nibbleChars, hiNibble) == -1 )
            throw new ArgumentException(string.Format("Non-hex character ({0}) present in input", hiNibble), "hiNibble");

        if( Array.IndexOf(nibbleChars, lowNibble) == -1 )
            throw new ArgumentException(string.Format("Non-hex character ({0}) present in input", lowNibble), "lowNibble");

        if( hiNibble >= 'a' )
            b = (byte)(10 + (hiNibble - 'a'));
        else
            b = (byte)(hiNibble - '0');

        b <<= 4;

        if( lowNibble >= 'a' )
            b |= (byte)(10 + (lowNibble - 'a'));
        else
            b |= (byte)(lowNibble - '0');
        
        return(b);
    }

    static void SaveAsBinary( string text, string fileName )
    {
        FileStream fileStream = File.Create(fileName);
        BinaryWriter fileWriter = new BinaryWriter(fileStream);
        char[] chars = text.ToCharArray();

        if( (chars.Length % 2) != 0 )
            throw new ArgumentException("Input text not divisible by 2");

        for( int n = 0; n < chars.Length; n += 2 )
        {
            fileWriter.Write(CharsToByte(chars[n], chars[n+1]));
        }

        fileWriter.Flush();
        fileWriter.Close();
        fileStream.Close();
    }

    static readonly char[] WHITESPACE = new char[]
    {
        // This list was pulled from the String.TrimEnd docs.
        //
        '\x9', '\xA', '\xB', '\xC', '\xD', '\x20', '\xA0',
        '\x2000', '\x2001', '\x2002', '\x2003', '\x2004',
        '\x2005', '\x2006', '\x2007', '\x2008', '\x2009', '\x200A',
        '\x200B', '\x3000', '\xFEFF', '\r', '\n'
    };

    static string FetchTextFromClipboard()
    {
        // Reads the contents of the clipboard, strips out all whitespace,
        // and returns the result.
        //
        IDataObject ClipboardData = Clipboard.GetDataObject();

        if( !ClipboardData.GetDataPresent(DataFormats.Text) )
            return("");

        return StripWhitespace(ClipboardData.GetData(DataFormats.Text) as string);
    }

    static string FetchTextFromFile( string fileName )
    {
        StreamReader r = File.OpenText(fileName);
        return StripWhitespace(r.ReadToEnd());
    }

    static string StripWhitespace( string text )
    {
        if( text == null )
            return("");

        char[] chars = text.ToCharArray();
        int IndexOfWhitespace;
        
        do
        {
            // Find the next whitespace char in the string.
            //
            IndexOfWhitespace = IndexOfAny(chars, WHITESPACE);

            if( IndexOfWhitespace != -1 )
            {
                // Find the next non-whitespace char in the string.
                //
                int IndexOfNonWhitespace = IndexOfWhitespace;

                while( Array.IndexOf(WHITESPACE, chars[IndexOfNonWhitespace]) != -1 )
                {
                    IndexOfNonWhitespace++;
                }

                // Remove all of the intervening whitespace.
                //
                int WhiteSpaceCount = IndexOfNonWhitespace - IndexOfWhitespace;
                chars = Remove(chars, IndexOfWhitespace, WhiteSpaceCount);
            }
        }
        while( IndexOfWhitespace != -1 );

        return(new string(chars));
    }

    static int IndexOfAny( char[] searchTarget, char[] tokens )
    {
        return IndexOfAny(searchTarget, 0, tokens);
    }

    static int IndexOfAny( char[] searchTarget, int startIndex, char[] tokens )
    {
        int index = -1;
        
        for( int n = startIndex; (index == -1) && (n < searchTarget.Length); n++ )
        {
            if( Array.IndexOf(tokens, searchTarget[n]) != -1 )
            {
                index = n;
            }
        }

        return(index);
    }

    static char[] Remove( char[] data, int startIndex, int charCount )
    {
        int charsToRight = data.Length - startIndex - charCount;

        if( charsToRight > 0 )
        {
            char[] newChars = new char[data.Length - charCount];
            Array.Copy(data, 0, newChars, 0, startIndex);
            Array.Copy(data, startIndex + charCount, newChars, startIndex, charsToRight);
            data = newChars;
        }

        return(data);
    }
}